-- creates a popup dialog with a prompt and input box
-- when the user hits "enter" in the input box it closes the dialog and runs the InputOK function
-- if the user hits "esc" in the input box or otherwise closes the dialog it simply cancels (closes dialog)

local importPath=string.gsub(getfenv(1)._.Name,"%.InputPrompt","").."."
local resourcePath=string.gsub(importPath,"%.","/").."Resources/"

import (importPath.."FontSupport");

InputPrompt = class(Turbine.UI.Lotro.Window);
function InputPrompt:Constructor(Title,Message,Font,ReturnFunction,CancelFunction)
	if Font==nil then Font=Turbine.UI.Lotro.Font.Verdana20 end
	self.Font=Font
	fontHeight=Turbine.UI.Lotro.FontInfo[Font].size
	self.fontMetric=FontMetric()
	self.fontMetric:SetParent(self)
	self.fontMetric:SetVisible(false)
	self.fontMetric:SetFont(Font)
	self.ReturnFunction=ReturnFunction
	self.CancelFunction=CancelFunction
	self.IsCancelled=false

	Turbine.UI.Lotro.Window.Constructor( self );
	local charWidth=8.3 -- rough estimate, but it'll have to do
	self:SetText(Title);
	self:SetVisible(true);
	local titleWidth=self.fontMetric:GetTextWidth(Title,fontHeight)

	local maxWidth=math.floor(Turbine.UI.Display:GetWidth()*.75);
	local maxHeight=math.floor(Turbine.UI.Display:GetHeight()*.75);
	if Message==nil then Message="" end
	local messageWidth=self.fontMetric:GetTextWidth(Message,fontHeight)+8 -- add 8 for control trim
	if messageWidth>maxWidth then messageWidth=maxWidth end;
	if messageWidth<titleWidth+140 then messageWidth=titleWidth+140 end
	if messageWidth<300 then messageWidth=300 end;
	local messageHeight=self.fontMetric:GetTextHeight(Message,messageWidth)+8
	if messageHeight>maxHeight then messageHeight=maxHeight end

	if fontColor==nil then fontColor=Turbine.UI.Color(1,1,1) end
	if backColor==nil then backColor=Turbine.UI.Color(0,0,0) end
	self.fontColor=fontColor
	self.backColor=backColor

	self.Loading=true
	self.Message=Turbine.UI.Label()
	self.Message:SetParent(self)
	self.Message:SetFont(self.Font)
	self.Message:SetForeColor(fontColor)
	self.Message:SetSize(messageWidth,messageHeight)
	self.Message:SetSelectable(true)
	self.Message:SetMarkupEnabled(true)
	self.Message:SetTextAlignment( Turbine.UI.ContentAlignment.MiddleLeft)
	self.Message:SetText(Message)
	self.Message.VScroll=Turbine.UI.Lotro.ScrollBar()
	self.Message.VScroll:SetOrientation(Turbine.UI.Orientation.Vertical)
	self.Message.VScroll:SetParent(self)
	self.Message.VScroll:SetBackColor(backColor)
	self.Message.VScroll:SetPosition(self.Message:GetLeft()+self.Message:GetWidth(),self.Message:GetTop())
	self.Message.VScroll:SetWidth(10)
	self.Message.VScroll:SetHeight(self.Message:GetHeight());
	self.Message:SetVerticalScrollBar(self.Message.VScroll)
	self.Message:SetPosition(10,40);

	self.Input=Turbine.UI.Lotro.TextBox();
	self.Input:SetParent(self)
	self.Input:SetFont(self.Font)
	self.Input:SetSize(self.Message:GetWidth(),fontHeight+4)
	self.Input:SetMultiline(false)
	self.Input:SetPosition(10,self.Message:GetTop()+self.Message:GetHeight()+10)
	self.Input.KeyDown=function(sender,args)
		if args.Action==162 then
			-- user hit Enter
			self.OK.Click()
		elseif args.Action==145 then
			self.Cancel.Click()
			-- user hit Esc
		end
	end

	self.OK=Turbine.UI.Lotro.Button()
	self.OK:SetParent(self)
	self.OK:SetSize(100,20)
	self.OK:SetPosition(45,self.Input:GetTop()+self.Input:GetHeight()+10)
	local tmpLanguage=Turbine.Engine:GetLanguage()
	if tmpLanguage==Turbine.Language.French then
		self.OK:SetText("D'accord")
	elseif tmpLanguage==Turbine.Language.German then
		self.OK:SetText("Okay")
	elseif tmpLanguage==Turbine.Language.Russian then
		self.OK:SetText("\209\133\208\190\209\128\208\190\209\136\208\190")
	else
		self.OK:SetText("OK")
	end
	self.OK.Click=function()
		self.Value=self.Input:GetText()
		if self.ReturnFunction~=nil then
			self.ReturnFunction()
		end
		self:SetVisible(false)
	end

	self.Cancel=Turbine.UI.Lotro.Button()
	self.Cancel:SetParent(self)
	self.Cancel:SetSize(self.OK:GetWidth(),20)
	self.Cancel:SetPosition(self.Message:GetWidth()-self.Cancel:GetWidth()-15,self.Input:GetTop()+self.Input:GetHeight()+10)
	local tmpLanguage=Turbine.Engine:GetLanguage()
	if tmpLanguage==Turbine.Language.French then
		self.Cancel:SetText("Annuler")
	elseif tmpLanguage==Turbine.Language.German then
		self.Cancel:SetText("Abbrechen")
	elseif tmpLanguage==Turbine.Language.Russian then
		self.Cancel:SetText("\208\158\209\130\208\188\208\181\208\189\208\176")
	else
		self.Cancel:SetText("Cancel")
	end
	self.Cancel.Click=function()
		if self.CancelFunction~=nil then
			self.CancelFunction()
		end
		self:SetVisible(false)
	end

	self:SetSize(self.Message:GetWidth()+30,self.Message:GetHeight()+self.Input:GetHeight()+self.OK:GetHeight()+85)

	self:SetPosition((Turbine.UI.Display:GetWidth() - self:GetWidth()) / 2,(Turbine.UI.Display:GetHeight() - self:GetHeight()) / 2);

	self.Update=function() -- I don't recall the need for this... something about the zorder not being accurate when first constructed possibly a bug in the API that no longer exists?
		if self.Loading then
			self:SetWantsUpdates(false);
			self.Loading=false;
			self:SetZOrder(self:GetZOrder()+1);
			self.Input:Focus()
		end
	end
	self:SetWantsUpdates(true);
end
