-- this plugin is used to dynamically load annotations and send them via chat commands to TerrainMap
-- the list of cells is passed via chat commands of the format "0TMCL_r_v_h" where r=region, v=cell v index, h=cell h index

local importPath=string.gsub(getfenv(1)._.Name,"%.TMAnnot","").."."
--import (importPath.."Table");
import "Turbine";
import "Turbine.UI";
--import "Turbine.UI.Lotro";
import (importPath.."VindarPatch");

string.encode=function(str)
	-- replace all non-alpha characters with a % escaped hex code
	local ret=""
	for i,v in ipairs({str:byte(1,-1)}) do
		if (v < 48) or (v>57 and v<65) or (v>90 and v<97) or (v > 125) then
			ret = ret .. "%" .. string.sub("00"..string.format("%x",tonumber(v)),-2)
		else
			ret = ret .. string.char(v)
		end
	end
	return ret
end
string.decode=function(str)
	local ret=""
	while string.len(str)~=0 do
		local char=string.sub(str,1,1)
		if char~="%" then
			ret=ret..char
			str=string.sub(str,2,-1)
		else
			local char=string.char(tonumber("0x"..string.sub(str,2,3)))
			ret=ret..char
			str = string.sub(str,4,-1)
		end
	end
	return ret;
end
string.split=function(str,separator)
	local tmpArray={};
	local tmpElem;
	str=tostring(str);
	if string.find("^$()%.[]*+-?",separator,1,true)~=nil then separator="%"..separator end
	for tmpElem in string.gmatch(str,"[^"..separator.."]*") do
		table.insert(tmpArray,tmpElem);
	end
	local tmpIndex=1;
	while tmpIndex<=#tmpArray do
		if tmpIndex==#tmpArray and tmpArray[tmpIndex]=="" then
			table.remove(tmpArray,tmpIndex);
		else
			if tmpArray[tmpIndex]=="" and tmpArray[tmpIndex+1]~="" then
				table.remove(tmpArray,tmpIndex);
			else
				tmpIndex=tmpIndex+1;
			end
		end
	end
	return tmpArray;
end

local cmdCount=0
local parms;
cellList={}
cmdList={}
local cmds=Turbine.Shell.GetCommands();
if cmds~=nil and type(cmds)=="table" then
	table.sort(cmds,function(arg1,arg2)if arg1<arg2 then return(true) end end);
	for cmdIndex=0,#cmds do
		if cmds[cmdIndex]~=nil then
			if cmdIndex>1 and cmds[cmdIndex]>="1" then
				break
			else
				parms=string.match(cmds[cmdIndex],"0TMCL_(.*)");
				if parms~=nil then
					table.insert(cellList,parms)
				end
			end
		end
	end
end
_G.AnnotData={}
_G.CustData={}
if #cellList==0 then
	-- user loaded automatically, just error out
	error("Do NOT run TMAnnot. This plugin is used internally by MoorMap and should not be started automatically or by the user.",0)
end
for k,v in ipairs(cellList) do
	-- try to process the file importPath.."\Annots\TM_Data_"..v
	local success,result=pcall(import,importPath.."Annots.TM_Data_"..v)
	if success then
		-- there were annotations found, now in "_G.AnnotData"
	end
-- now load custom annots if any
	local custData=PatchDataLoad(Turbine.DataScope.Account, "TM_Cust_Annot_"..v)
	if custData~=nil then
		for k,v in ipairs(custData) do
			table.insert(_G.CustData,v)
		end
	end
end
for k,v in ipairs(_G.AnnotData) do
	-- now create a "0TMA_" command object for each annotation
	-- note, the annot data is already string encoded to avoid conflicts and save time
	local tmpCmd=Turbine.ShellCommand()
	table.insert(cmdList,tmpCmd)
	Turbine.Shell.AddCommand("0TMA_D_"..string.encode(v),tmpCmd)
end
if _G.CustData~=nil then
	for k,v in ipairs(_G.CustData) do
		-- now create a "0TMA_" command object for each annotation
		-- note, the annot data is already string encoded to avoid conflicts and save time
		local tmpCmd=Turbine.ShellCommand()
		table.insert(cmdList,tmpCmd)
		Turbine.Shell.AddCommand("0TMA_C_"..string.encode(v),tmpCmd)
	end
end
plugin.Unload = function(sender,args)
	-- remove the commands
	for k,v in pairs(cmdList) do
		Turbine.Shell.RemoveCommand(v)
	end
	-- clean up last set of annotations
	_G.AnnotData=nil
end
local tmpCmd=Turbine.ShellCommand()
table.insert(cmdList,tmpCmd)
Turbine.Shell.AddCommand("0TMAC",tmpCmd)

