-- this plugin is used to dynamically update custom annotation .plugindata files
-- the list of cells is passed via chat commands of the format "0TMCD_action_data" where action='A' to add or 'D' to delete the annotation and data is an encoded data string to be added/updated/removed in the file
-- -- note, to 'update' an annotation you must delete the old and add the new since the encoded string has no other identifier and must match exactly

local importPath=string.gsub(getfenv(1)._.Name,"%.TMCust","").."."
--import (importPath.."Table")
import "Turbine"
import "Turbine.UI"
--import "Turbine.UI.Lotro"
import (importPath.."VindarPatch")
import (importPath.."MapInfo")
import (importPath.."Class")
import (importPath.."Table")

string.encode=function(str)
	-- replace all non-alpha characters with a % escaped hex code
	local ret=""
	for i,v in ipairs({str:byte(1,-1)}) do
		if (v < 48) or (v>57 and v<65) or (v>90 and v<97) or (v > 125) then
			ret = ret .. "%" .. string.sub("00"..string.format("%x",tonumber(v)),-2)
		else
			ret = ret .. string.char(v)
		end
	end
	return ret
end
string.decode=function(str)
	local ret=""
	while string.len(str)~=0 do
		local char=string.sub(str,1,1)
		if char~="%" then
			ret=ret..char
			str=string.sub(str,2,-1)
		else
			local char=string.char(tonumber("0x"..string.sub(str,2,3)))
			ret=ret..char
			str = string.sub(str,4,-1)
		end
	end
	return ret;
end
string.split=function(str,separator)
	local tmpArray={};
	local tmpElem;
	str=tostring(str);
	if string.find("^$()%.[]*+-?",separator,1,true)~=nil then separator="%"..separator end
	for tmpElem in string.gmatch(str,"[^"..separator.."]*") do
		table.insert(tmpArray,tmpElem);
	end
	local tmpIndex=1;
	while tmpIndex<=#tmpArray do
		if tmpIndex==#tmpArray and tmpArray[tmpIndex]=="" then
			table.remove(tmpArray,tmpIndex);
		else
			if tmpArray[tmpIndex]=="" and tmpArray[tmpIndex+1]~="" then
				table.remove(tmpArray,tmpIndex);
			else
				tmpIndex=tmpIndex+1;
			end
		end
	end
	return tmpArray;
end

Grid={}
Grid[1]={}
Grid[2]={}
Grid[3]={}
Grid[4]={}
Grid[1].NSOffset=26
Grid[1].EWOffset=-110.75
Grid[2].NSOffset=47.62
Grid[2].EWOffset=-119.6
Grid[3].NSOffset=-15.6
Grid[3].EWOffset=-96.4
Grid[4].NSOffset=-15.6
Grid[4].EWOffset=-25.2

Grid[1].YOffSet=37400
Grid[1].XOffSet=9200
Grid[2].YOffSet=42800
Grid[2].XOffSet=7000
Grid[3].YOffSet=21200
Grid[3].XOffSet=12800
Grid[3].YOffSet=27000
Grid[3].XOffSet=12800
Grid[4].YOffSet=27000
Grid[4].XOffSet=30600

GetNSEWfromHVXY=function(region,h,v,x,y)
	local NS=math.floor((Grid[region].NSOffset-((v+y/200)*.8))*100+.5)/100
	local EW=math.floor((Grid[region].EWOffset+((h+x/200)*.8))*100+.5)/100
	return {["NS"]=NS,["EW"]=EW}
end

local cmdCount=0
local parms;
cellList={}
cmdList={}
local cmds=Turbine.Shell.GetCommands();
if cmds~=nil and type(cmds)=="table" then
	table.sort(cmds,function(arg1,arg2)if arg1<arg2 then return(true) end end);
	for cmdIndex=0,#cmds do
		if cmds[cmdIndex]~=nil then
			if cmdIndex>1 and cmds[cmdIndex]>="1" then
				break
			else
				parms=string.match(cmds[cmdIndex],"0TMCD_(.*)");
				if parms~=nil then
					table.insert(cellList,{string.sub(parms,1,1),string.sub(parms,3)})
				end
			end
		end
	end
end
_G.AnnotData={}
local region,row,col
if #cellList==0 then
	-- user loaded automatically, just error out
	error("Do NOT run TMCust. This plugin is used internally by MoorMap and should not be started automatically or by the user.",0)
end

--update cellList resolving row2, col2, ns and ew
for k,v in ipairs(cellList) do
	-- v[1]="S" to save entry or "D" to delete entry
	local cd=string.split(v[2],"_")
	row2=math.floor((cd[3]-1)/4)*4+1
	cellList[k].region=tonumber(cd[1])
	cellList[k].row=math.floor((tonumber(cd[3])-1)/4)*4+1
	cellList[k].col=math.floor((tonumber(cd[2])-1)/4)*4+1
	local nsew=GetNSEWfromHVXY(tonumber(cd[1]),tonumber(cd[2]),tonumber(cd[3]),tonumber(cd[4]),tonumber(cd[5]))
	cellList[k].ns=nsew.NS
	cellList[k].ew=nsew.EW
	cellList[k].type=tonumber(cd[6])
	if cd[9]==nil then
		cellList[k].oldType=0
	else
		cellList[k].oldType=tonumber(cd[9])
	end

	cellList[k].newMMName=string.decode(cd[7])..": "..string.decode(cd[8])
	-- cd[10-11] are optional original values (for delete and update)
	if cd[10]==nil then
		cd[10]=""
	end
	if cd[11]==nil then cd[11]=cd[10] end
	cellList[k].oldMMName=string.decode(cd[10])..": "..string.decode(cd[11])
	if v[1]=="D" or v[1]=="S" then
		cellList[k].oldEntry=cd[1].."_"..cd[2].."_"..cd[3].."_"..cd[4].."_"..cd[5].."_"..cd[6].."_"..cd[7].."_"..cd[8]
	else
		cellList[k].oldEntry=cd[1].."_"..cd[2].."_"..cd[3].."_"..cd[4].."_"..cd[5].."_"..cd[9].."_"..cd[10].."_"..cd[11]
	end
	cellList[k][2]=cd[1].."_"..cd[2].."_"..cd[3].."_"..cd[4].."_"..cd[5].."_"..cd[6].."_"..cd[7].."_"..cd[8] -- strip off the last two values
end
table.sort(cellList,function(a,b) if a.region<b.region then return (true) else if a.region==b.region and a.row<b.row then return (true) else if a.region==b.region and a.row==b.row and a.col<b.col then return (true) end end end end)
MMList={}
for k,v in ipairs(cellList) do
	local oldIndex,newIndex
	-- v[1]="S" to save entry or "D" to delete entry
	if region==nil or region~=v.region or row~=v.row or col~=v.col then
		if region~=nil and custAnnot~=nil then
			-- save current annots
			PatchDataSave(Turbine.DataScope.Account, "TM_Cust_Annot_"..region.."_"..row.."_"..col,custAnnot)
		end
		-- load existing custom annots if any
		custAnnot=PatchDataLoad(Turbine.DataScope.Account, "TM_Cust_Annot_"..v.region.."_"..v.row.."_"..v.col)
		if custAnnot==nil then custAnnot={} end
	end
	region=v.region
	row=v.row
	col=v.col
	-- scan custAnnot looking for a match to v[2]
	oldIndex=nil
	newIndex=nil
	for i,j in ipairs(custAnnot) do
		if j==v.oldEntry then
			oldIndex=i
		end
		if j==v[2] then
			newIndex=i
		end
		if oldIndex~=nil and newIndex~=nil then
			break
		end
	end
	if v[1]=="D" then
		if oldIndex~=nil then
			-- remove the matching entry
			table.remove(custAnnot,oldIndex)
		end
	elseif v[1]=="U" then
		if oldIndex~=nil and (newIndex==nil or oldIndex~=newIndex) then
			-- remove the matching entry
			table.remove(custAnnot,oldIndex)
		end
		if newIndex==nil then
			table.insert(custAnnot,v[2])
		end
	else
		-- fwiw, in the case of "S", oldIndex and newIndex should always be the same
		if newIndex==nil then
			-- only save new entry if there isn't an identical match already
			table.insert(custAnnot,v[2])
		end
	end
	-- get table of mapIDs that match coords and fill a new table with action, mapid, ns, ew, type, "name: desc" for updating MM
--*** match ANY map that contains the coords that is zoom > 2 (no sense allowing custom annots on regional maps)
	for i,m in ipairs(mapInfo) do
		if m[3]==1 and m[4]==v.region and m[9]>2 then
			if m[5]>=v.ns and m[6]<=v.ew and m[7]<=v.ns and m[8]>=v.ew then
				table.insert(MMList,{v[1], i,v.ns, v.ew, v.type, v.oldType, v.newMMName, v.oldMMName})
			end
		end
	end
end
if region~=nil and custAnnot~=nil then
	-- save current annots
	PatchDataSave(Turbine.DataScope.Account, "TM_Cust_Annot_"..region.."_"..row.."_"..col,custAnnot);
end
table.sort(MMList, function(a,b) if a[2]<b[2] then return(true) end end) -- only bother sorting by mapid since that determines file
local mapID=nil
local MMAnnots={}
for k,v in ipairs(MMList) do
	if mapID==nil or mapID~=v[2] then
		if mapID~=nil and MMAnnots~=nil then
			PatchDataSave(Turbine.DataScope.Account, "MM_CustomAnnotations_"..tostring(mapID),MMAnnots)
		end
		MMAnnots=PatchDataLoad(Turbine.DataScope.Account, "MM_CustomAnnotations_"..tostring(v[2]))
		if MMAnnots==nil then MMAnnots={} end
	end
	mapID=v[2]
	local oldIndex=nil
	local newIndex=nil
	for i,a in ipairs(MMAnnots) do
		-- find entry that matches original value, only need to test ns, ew and name (if map matched then region already matched)
	--v{[1]="action",[2]=map,[3]="ns", [4]="ew", [5]="type", [6]=oldType, [7]="newname: newdesc",[8]="oldname: olddesc"}
	--a{["1"]=map, ["2"]=ns, ["3"]=ew, ["4"]="type",["5"]="allegiance",["6"]=nil,["7"]="name: desc",[8-11]=0}

		if a[2]==v[3] and a[3]==v[4] then
			if a[4]==v[6] and a[7]==v[8] then
				oldIndex=i
			end
			if a[4]==v[5] and a[7]==v[7] then
				newIndex=i
			end
			if oldIndex~=nil and newIndex~=nil then
				break
			end
		end
	end

	if v[1]=="D" then
		-- for deletions we use the new info as if it was old
		if newIndex~=nil then
			table.remove(MMAnnots,newIndex)
		end
	elseif v[1]=="U" then
		-- can't depend on old existing since user may have caused synch issues via annotation maint in moormap, so, if an old version is found, just delete it and insert a new one.
		local allegiance=0 -- allegiance isn't supported by terrainmap so ony preserve it for updates
		if oldIndex~=nil and (newIndex==nil or oldIndex~=newIndex) then
			allegiance=MMAnnots[oldIndex][5]
			table.remove(MMAnnots,oldIndex)
		end
		if newIndex==nil then
			table.insert(MMAnnots,{v[2],v[3],v[4],v[5],allegiance,nil,v[7]})
		end
	else
		if newIndex==nil then
			table.insert(MMAnnots,{v[2],v[3],v[4],v[5],0,nil,v[7]})
		end
	end
end
if mapID~=nil and MMAnnots~=nil then
	PatchDataSave(Turbine.DataScope.Account, "MM_CustomAnnotations_"..tostring(mapID),MMAnnots)
end

plugin.Unload = function(sender,args)
	-- remove the 'done' command
	Turbine.Shell.RemoveCommand(tmpCmd)
end
tmpCmd=Turbine.ShellCommand()
Turbine.Shell.AddCommand("0TMCC",tmpCmd) -- signal that we are done
