
local importPath=string.gsub(getfenv(1)._.Name,"%.TMSearch","").."."
local resourcePath=string.gsub(importPath,"%.","/").."Resources/"
-- This plugin gets it's parameters from the "0TMP_" command (registered by TerrainMap before this plugin is loaded)
-- this plugin then processes each map's data file, searching for the parameter and creating a "0TMR_" result command for each match
-- once done, it creates the "0TMC" command which causes TerrainMap to process and "0TMR_" commands and then unload this plugin
-- in this plugin's unload, the "0TMR_" commands and "0TMC" are removed since TerrainMap processes them before unloading this plugin

import (importPath.."Class");
import (importPath.."Table");
import (importPath.."MapInfo");

import "Turbine";
import "Turbine.UI";
import "Turbine.UI.Lotro";

local cmdCount=0
local parms
local cmds=Turbine.Shell.GetCommands();
if cmds~=nil and type(cmds)=="table" then
	table.sort(cmds,function(arg1,arg2)if arg1<arg2 then return(true) end end);
	for cmdIndex=0,#cmds do
		if cmds[cmdIndex]~=nil then
			if cmdIndex>1 and cmds[cmdIndex]>="1" then
				break
			else
				parms=string.match(cmds[cmdIndex],"0TMP_(.*)");
				if parms~=nil then
					break
				end
			end
		end
	end
end
if parms==nil then
	error("Do NOT run TMSearch. This plugin is used internally by TerrainMap and should not be started automatically or by the user.",0)
end
_G.tmpAnnotations={}
_G.tmpMapString={}

respCmd={}
replaceStr={[128]="A",[129]="A",[130]="A",[131]="A",[132]="A",[133]="A",[134]="AE",[135]="C",[136]="E",[137]="E",[138]="E",[139]="E",[140]="I",[141]="I",[142]="I",[143]="I",[144]="",[145]="N",[146]="O",[147]="O",[148]="O",[149]="O",[150]="O",[151]="",[152]="O",[153]="U",[154]="U",[155]="U",[156]="U",[157]="Y",[158]="Y",[159]="sz",[160]="a",[161]="a",[162]="a",[163]="a",[164]="a",[165]="a",[166]="ae",[167]="c",[168]="e",[169]="e",[170]="e",[171]="e",[172]="i",[173]="i",[174]="i",[175]="i",[176]="o",[177]="n",[178]="o",[179]="o",[180]="o",[181]="o",[182]="o",[183]="",[184]="o",[185]="u",[186]="u",[187]="u",[188]="u",[189]="y",[190]="y",[191]="y"}
string.stripaccent=function(str)
	local ret = "";
	if str~=nil then
		ret=""
		local replace=false
		for i,v in ipairs({str:byte(1,-1)}) do
			if replace then
				replace=false
				if replaceStr[v]~=nil then
					ret=ret..replaceStr[v]
				end
			else
				if (v==195) then
					replace=true
				else
					ret = ret .. string.char(v);
				end
			end
		end
	end
	return ret;
end
string.split=function(str,separator)
	local tmpArray={};
	local tmpElem;
	str=tostring(str);
	if string.find("^$()%.[]*+-?",separator,1,true)~=nil then separator="%"..separator end
	for tmpElem in string.gmatch(str,"[^"..separator.."]*") do
		table.insert(tmpArray,tmpElem);
	end
	local tmpIndex=1;
	while tmpIndex<=#tmpArray do
		if tmpIndex==#tmpArray and tmpArray[tmpIndex]=="" then
			table.remove(tmpArray,tmpIndex);
		else
			if tmpArray[tmpIndex]=="" and tmpArray[tmpIndex+1]~="" then
				table.remove(tmpArray,tmpIndex);
			else
				tmpIndex=tmpIndex+1;
			end
		end
	end
	return tmpArray;
end
searchName=string.lower(string.stripaccent(parms))

statusDisplay=Turbine.UI.Lotro.Window();
statusDisplay:SetZOrder(3)
statusDisplay:SetSize(400,100);
statusDisplay:SetPosition((Turbine.UI.Display:GetWidth() - statusDisplay:GetWidth())/2,(Turbine.UI.Display:GetHeight() - statusDisplay:GetHeight())/2)
statusDisplay:SetText("TerrainMap Search");
statusDisplay.Status="Loading";
statusDisplay.StatusText=Turbine.UI.Label();
statusDisplay.StatusText:SetParent(statusDisplay);
statusDisplay.StatusText:SetSize(200,18);
statusDisplay.StatusText:SetPosition((statusDisplay:GetWidth()-statusDisplay.StatusText:GetWidth())/2,40);
statusDisplay.StatusText:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter);
statusDisplay.StatusText:SetText(statusDisplay.Status);

statusDisplay.MapText=Turbine.UI.Label();
statusDisplay.MapText:SetParent(statusDisplay);
statusDisplay.MapText:SetSize(200,18);
statusDisplay.MapText:SetPosition((statusDisplay:GetWidth()-statusDisplay.MapText:GetWidth())/2,60);
statusDisplay.MapText:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter);

statusDisplay:SetWantsUpdates(true);
statusDisplay.MapIndex=1;
statusDisplay.WorldIndex=0;

statusDisplay.Update=function()
	if (Plugins["TMSearch"] ~= nil) and statusDisplay.Status=="Loading" then
		statusDisplay.Status="Initializing";
		Plugins["TMSearch"].Unload = function(statusDisplay,sender,args)
			-- remove the "0TMR_" commands and the "0TMC" command
			for k,v in pairs(respCmd) do
				Turbine.Shell.RemoveCommand(v)
			end
			-- clean up last set of annotations
			_G.tmpAnnotations=nil
		end
	elseif statusDisplay.Status=="Initializing" then
		statusDisplay.Status="Searching"
	elseif statusDisplay.Status=="Searching" then
		if statusDisplay.MapIndex>#mapInfo then
			statusDisplay.Status="Done"
			statusDisplay.MapText:SetText("");
		else
			statusDisplay.MapText:SetText(mapInfo[statusDisplay.MapIndex][1]);
			ProcessMap();
			statusDisplay.MapIndex=statusDisplay.MapIndex+1
		end
	elseif statusDisplay.Status=="Done" then
		statusDisplay:SetWantsUpdates(false);
-- trigger our unload by registering the command "0TMC"
		cmdCount=cmdCount+1
		respCmd[cmdCount]=Turbine.ShellCommand();
		Turbine.Shell.AddCommand("0TMC",respCmd[cmdCount]);
	end		
	statusDisplay.StatusText:SetText(statusDisplay.Status);
end


function ProcessMap()
	local result,error;
	local success,retval;
	result,error=loadstring("import \"GaranStuff.TerrainMap.Data.MapData_"..statusDisplay.MapIndex.."\"");
	if result==nil then
		Turbine.Shell.WriteLine("Error:"..tostring(error));
	else
		-- we may not have a data file for all maps (some have no annotations)
		success,retval=pcall(result);
		if success then
			for k,v in ipairs(_G.tmpAnnotations) do
				if string.find(string.lower(string.stripaccent(_G.tmpMapString[1][v[6]])),searchName)~=nil then
					local tmpStr=string.split(tmpMapString[1][v[6]],":")
					cmdCount=cmdCount+1
					respCmd[cmdCount]=Turbine.ShellCommand();
					if tmpStr[1]~=nil then
						tmpStr[1]=string.gsub(tmpStr[1],",","\194\184")
					end
					if tmpStr[2]~=nil then
						tmpStr[2]=string.gsub(tmpStr[2],",","\194\184")
					end
					if mapInfo[statusDisplay.MapIndex][10]==nil then
						Turbine.Shell.AddCommand("0TMR_"..tostring(cmdCount).."_"..tostring(mapInfo[statusDisplay.MapIndex][4]).."_"..tostring(v[2]).."_"..tostring(v[3]).."_"..tostring(tmpStr[1]).."_"..tostring(mapInfo[statusDisplay.MapIndex][1]).."_"..tostring(mapInfo[statusDisplay.MapIndex][9]).."_"..tostring(tmpStr[2]),respCmd[cmdCount]);
					else
						Turbine.Shell.AddCommand("0TMR_"..tostring(cmdCount).."_"..tostring(mapInfo[statusDisplay.MapIndex][4]).."_"..tostring(mapInfo[statusDisplay.MapIndex][10]).."_"..tostring(mapInfo[statusDisplay.MapIndex][11]).."_"..tostring(tmpStr[1]).."_"..tostring(mapInfo[statusDisplay.MapIndex][1]).."_"..tostring(mapInfo[statusDisplay.MapIndex][9]).."_"..tostring(tmpStr[2]),respCmd[cmdCount]);
					end
				end
			end
			return retval;
		else
--			Turbine.Shell.WriteLine("Error:"..tostring(retval))
			return nil;
		end
	end
end

statusDisplay:SetZOrder(statusDisplay:GetZOrder()+1);
statusDisplay:SetVisible(true);